==> letsencrypt-8.x-1.10.tar.gz: letsencrypt/src/Service/LetsEncrypt.php <==
<?php

namespace Drupal\letsencrypt\Service;

use Drupal\idna\Service\IdnaConvertInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use LEClient\LEClient;
use LEClient\LEOrder;

/**
 * Class LetsEncrypt.
 */
class LetsEncrypt implements LetsEncryptInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Config factory.
   *
   * @var \Drupal\idna\Service\IdnaConvertInterface
   */
  protected $idna;

  /**
   * Creates a new Lescript.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\idna\Service\IdnaConvertInterface $idna
   *   Idna Convert.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
      IdnaConvertInterface $idna) {
    // Services.
    $config = $config_factory->get('letsencrypt.settings');
    $this->configFactory = $config_factory;
    $this->idna = $idna;
    // Configs.
    $this->type = LEOrder::CHALLENGE_TYPE_HTTP;
    $this->mode = $config->get('acme-url');
    $this->email = $config->get('cert-email');
    $this->logLevel = $config->get('acme-log');
    $this->basename = FALSE;
    // Cert & Accaunt Dir.
    $cdir = $this->prepareDir($config);
    $this->cdir = $cdir;
    $this->acc = [
      "private_key" => "$cdir/_account/private.pem",
      "public_key" => "$cdir/_account/public.pem",
    ];
    // Challenge.
    $this->challenge = DRUPAL_ROOT . '/.well-known/acme-challenge';
  }

  /**
   * Init.
   */
  public function init($type = LEOrder::CHALLENGE_TYPE_HTTP) {
    $this->type = $type;
    return $this;
  }

  /**
   * Mode: stage/prod.
   */
  public function setMode(string $mode = 'stage') {
    $this->mode = $mode;
    return $this;
  }

  /**
   * Get Mode URL: stage/prod.
   */
  public function getModeUrl() {
    $url = [
      'prod' => 'https://acme-v02.api.letsencrypt.org',
      'stage' => 'https://acme-staging-v02.api.letsencrypt.org',
    ];
    return $url[$this->mode];
  }

  /**
   * Mode: stage/prod.
   */
  public function setLogLevel(int $level = 0) {
    $this->logLevel = $level;
    return $this;
  }

  /**
   * Get Mode URL: stage/prod.
   */
  public function getLogLevel() {
    // Configs.
    $log = [
      // Logs no messages or faults, except Runtime Exceptions.
      'LOG_OFF' => 0,
      // Logs only messages and faults.
      'LOG_STATUS' => 1,
      // Logs messages, faults and raw responses from HTTP requests.
      'LOG_DEBUG' => 2,
    ];
    return $log[$this->logLevel];
  }

  /**
   * SET certificate directory.
   */
  public function setCertDir(string $cdir) {
    // $cflag = FileSystemInterface::CREATE_DIRECTORY;
    // \Drupal::service('file_system')->prepareDirectory($directory, $cflag); .
    $this->cdir = $cdir;
    $this->createDir("$cdir/_account");
    $this->acc = [
      "private_key" => "$cdir/_account/private.pem",
      "public_key" => "$cdir/_account/public.pem",
    ];
    return $this;
  }

  /**
   * SET Domains.
   */
  public function setExtra(array $extra) {
    $this->extra = $extra;
    return $this;
  }

  /**
   * SET Domains.
   */
  public function setDomains(array $domains) {
    $this->domains = array_map(function ($domain) {
      return \Drupal::service('idna')->encode($domain);
    }, $domains);
    if (!$this->basename) {
      $this->basename = reset($this->domains);
    }
    return $this;
  }

  /**
   * SET BaseName.
   */
  public function setBaseName(string $basename) {
    $this->basename = $basename;
    return $this;
  }

  /**
   * SET DirName.
   */
  public function setDirName(string $dirname) {
    $this->dirname = $dirname;
    return $this;
  }

  /**
   * Read Cert.
   */
  public function read($host) {
    $path = $this->cdir . "/$host";
    return $this->info($path);
  }

  /**
   * Read Cert.
   */
  public function info($dir) {
    $cert = FALSE;
    $private = "{$dir}/private.pem";
    $fullchain = "{$dir}/fullchain.pem";
    if (file_exists($fullchain)) {
      $cert_read = openssl_x509_read("file://$fullchain");
      $cert_info = openssl_x509_parse($cert_read);
      if ($cert_info && $cert_info['validTo_time_t'] > time()) {
        $days = round(($cert_info['validTo_time_t'] - time()) / 3600 / 24);
        $expire = date('d.M.Y', $cert_info['validTo_time_t']);
        $date = date('Y-m-d\TH:i:s', $cert_info['validTo_time_t']);
        $alt = str_replace(["DNS:", " "], "", $cert_info['extensions']['subjectAltName']);
        // array_map(fn($domain) => $idna->decode($domain), $names)); //.
        $names = array_map(function ($domain) {
          return \Drupal::service('idna')->decode($domain);
        }, explode(",", $alt));
        $cert = [
          'date' => $date,
          'days' => $days,
          'names' => $names,
          'expire' => $expire,
          'message' => "<b>{$days}</b> days left, expire at {$expire}",
          'private' => file_get_contents($private),
          'fullchain' => file_get_contents($fullchain),
        ];
      }
    }
    return $cert;
  }

  /**
   * Sign.
   */
  public function signPipe($callback = FALSE) {
    $cert = $this->cert($this->dirname);
    $client = new LEClient(
      $this->email,
      $this->getModeUrl(),
      $this->getLogLevel(),
      $cert,
      $this->acc
    );
    $account = $client->getAccount();
    // Order.
    $order = $client->getOrCreateOrder($this->basename, $this->domains);
    // Check whether there are any authorizations pending.
    // If that is the case, try to verify the pending authorizations.
    if (!$order->allAuthorizationsValid()) {
      // Get the HTTP challenges from the pending authorizations.
      $pending = $order->getPendingAuthorizations($this->type);
      // Do Acme Challenge.
      if (!empty($pending)) {
        foreach ($pending as $challenge) {
          // Wright challenge.
          if (!$callback) {
            $this->acmeChallenge($challenge);
          }
          else {
            $callback($challenge, $this->extra);
          }
          // Let LetsEncrypt verify this challenge.
          $order->verifyPendingOrderAuthorization($challenge['identifier'], $this->type);
        }
      }
    }
    // Check once more whether all authorizations are valid
    // before we can finalize the order.
    if ($order->allAuthorizationsValid()) {
      // Finalize the order first, if that is not yet done.
      if (!$order->isFinalized()) {
        $order->finalizeOrder();
      }
      // Check whether the order has been finalized before we can get
      // the certificate. If finalized, get the certificate.
      if ($order->isFinalized()) {
        $order->getCertificate();
        file_put_contents($cert['domains'], implode("\n", $this->domains));
        $date = time() + 60 * 60 * 24 * 90;
        $datetime = \Drupal::service('date.formatter')->format($date, 'custom', 'Y-m-d\TH:i:s');
        file_put_contents($cert['expire'], $datetime);
      }
    }
    return implode(", \n", $this->domains);
  }

  /**
   * Get Cert.
   */
  public function sign($basename, array $domains = []) {
    $all = [];
    $this->domains = array_map(function ($domain) {
      $domain = trim($domain);
      return \Drupal::service('idna')->encode($domain);
    }, $domains);
    $this->basename = $basename;
    $this->dirname = $basename;
    // $callback = $this->acmeChallenge();
    $this->signPipe();
    return implode(", \n", $this->domains);
  }

  /**
   * Acme Challenge.
   */
  private function acmeChallenge($challenge, $drupalroot = TRUE) {
    \Drupal::service('module_handler')->alter('letsencrypt_challenge', $challenge, $drupalroot);
    if ($drupalroot) {
      $dir = $this->challenge;
      $this->createDir($this->challenge, FALSE);
      file_put_contents("{$dir}/{$challenge['filename']}", $challenge['content']);
    }
  }

  /**
   * Prepare cert directory.
   */
  private function prepareDir($config) {
    $cdir = getenv("HOME") . "cert";
    if ($dir = $config->get('cert-dir')) {
      if (substr($dir, 0, '2') == '~/') {
        $dir = getenv("HOME") . substr($dir, 2);
      }
      if (substr($dir, 0, '2') == 'private://') {
        $dir = \Drupal::service('file_system')->realpath($dir);
      }
      $cdir = $dir;
    }
    $this->createDir($cdir);
    $this->createDir("$cdir/_account");
    return $cdir;
  }

  /**
   * Cert Path.
   */
  public function cert($base) {
    $ddir = \Drupal::transliteration()->transliterate($this->cdir . "/$base");
    $this->createDir($ddir);
    $cert = [
      "public_key" => "{$ddir}/public.pem",
      "private_key" => "{$ddir}/private.pem",
      "certificate" => "{$ddir}/certificate.crt",
      "fullchain_certificate" => "{$ddir}/fullchain.pem",
      "order" => "{$ddir}/order",
      "domains" => "{$ddir}/domains",
      "expire" => "{$ddir}/expire",
    ];
    return $cert;
  }

  /**
   * Create Directory.
   */
  private function createDir($directory, $protect = TRUE) {
    if (!file_exists($directory)) {
      mkdir($directory, 0777, TRUE);
      if ($protect) {
        file_put_contents("$directory/.htaccess", "order deny,allow\ndeny from all");
      }
    }
  }

}

==> letsencrypt-8.x-1.11.tar.gz: letsencrypt/src/Service/LetsEncrypt.php <==
<?php

namespace Drupal\letsencrypt\Service;

use Drupal\idna\Service\IdnaConvertInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use LEClient\LEClient;
use LEClient\LEOrder;

/**
 * Class LetsEncrypt.
 */
class LetsEncrypt implements LetsEncryptInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Config factory.
   *
   * @var \Drupal\idna\Service\IdnaConvertInterface
   */
  protected $idna;

  /**
   * Creates a new Lescript.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\idna\Service\IdnaConvertInterface $idna
   *   Idna Convert.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
      IdnaConvertInterface $idna) {
    // Services.
    $config = $config_factory->get('letsencrypt.settings');
    $this->configFactory = $config_factory;
    $this->idna = $idna;
    // Configs.
    $this->type = LEOrder::CHALLENGE_TYPE_HTTP;
    $this->mode = $config->get('acme-url');
    $this->email = $config->get('cert-email');
    $this->logLevel = $config->get('acme-log');
    $this->basename = FALSE;
    // Cert & Accaunt Dir.
    $cdir = $this->prepareDir($config);
    $this->cdir = $cdir;
    $this->acc = [
      "private_key" => "$cdir/_account/private.pem",
      "public_key" => "$cdir/_account/public.pem",
    ];
    // Challenge.
    $this->challenge = DRUPAL_ROOT . '/.well-known/acme-challenge';
  }

  /**
   * Init.
   */
  public function init($type = LEOrder::CHALLENGE_TYPE_HTTP) {
    $this->type = $type;
    return $this;
  }

  /**
   * Mode: stage/prod.
   */
  public function setMode(string $mode = 'stage') {
    $this->mode = $mode;
    return $this;
  }

  /**
   * Get Mode URL: stage/prod.
   */
  public function getModeUrl() {
    $url = [
      'prod' => 'https://acme-v02.api.letsencrypt.org',
      'stage' => 'https://acme-staging-v02.api.letsencrypt.org',
    ];
    return $url[$this->mode];
  }

  /**
   * Mode: stage/prod.
   */
  public function setLogLevel(string $level = 'LOG_OFF') {
    $this->logLevel = $level;
    return $this;
  }

  /**
   * Get Mode URL: stage/prod.
   */
  public function getLogLevel() {
    // Configs.
    $log = [
      // Logs no messages or faults, except Runtime Exceptions.
      'LOG_OFF' => 0,
      // Logs only messages and faults.
      'LOG_STATUS' => 1,
      // Logs messages, faults and raw responses from HTTP requests.
      'LOG_DEBUG' => 2,
    ];
    return $log[$this->logLevel];
  }

  /**
   * SET certificate directory.
   */
  public function setCertDir(string $cdir) {
    // $cflag = FileSystemInterface::CREATE_DIRECTORY;
    // \Drupal::service('file_system')->prepareDirectory($directory, $cflag); .
    $this->cdir = $cdir;
    $this->createDir("$cdir/_account");
    $this->acc = [
      "private_key" => "$cdir/_account/private.pem",
      "public_key" => "$cdir/_account/public.pem",
    ];
    return $this;
  }

  /**
   * SET Domains.
   */
  public function setExtra(array $extra) {
    $this->extra = $extra;
    return $this;
  }

  /**
   * SET Domains.
   */
  public function setDomains(array $domains) {
    $this->domains = array_map(function ($domain) {
      return \Drupal::service('idna')->encode($domain);
    }, $domains);
    if (!$this->basename) {
      $this->basename = reset($this->domains);
    }
    return $this;
  }

  /**
   * SET BaseName.
   */
  public function setBaseName(string $basename) {
    $this->basename = $basename;
    return $this;
  }

  /**
   * SET DirName.
   */
  public function setDirName(string $dirname) {
    $this->dirname = $dirname;
    return $this;
  }

  /**
   * Read Cert.
   */
  public function read($host) {
    $path = $this->cdir . "/$host";
    return $this->info($path);
  }

  /**
   * Read Cert.
   */
  public function info($dir) {
    $cert = FALSE;
    $private = "{$dir}/private.pem";
    $fullchain = "{$dir}/fullchain.pem";
    if (file_exists($fullchain)) {
      $cert_read = openssl_x509_read("file://$fullchain");
      $cert_info = openssl_x509_parse($cert_read);
      if ($cert_info && $cert_info['validTo_time_t'] > time()) {
        $days = round(($cert_info['validTo_time_t'] - time()) / 3600 / 24);
        $expire = date('d.M.Y', $cert_info['validTo_time_t']);
        $date = date('Y-m-d\TH:i:s', $cert_info['validTo_time_t']);
        $alt = str_replace(["DNS:", " "], "", $cert_info['extensions']['subjectAltName']);
        // array_map(fn($domain) => $idna->decode($domain), $names)); //.
        $names = array_map(function ($domain) {
          return \Drupal::service('idna')->decode($domain);
        }, explode(",", $alt));
        $cert = [
          'date' => $date,
          'days' => $days,
          'names' => $names,
          'expire' => $expire,
          'message' => "<b>{$days}</b> days left, expire at {$expire}",
          'private' => file_get_contents($private),
          'fullchain' => file_get_contents($fullchain),
        ];
      }
    }
    return $cert;
  }

  /**
   * Sign.
   */
  public function signPipe($callback = FALSE) {
    $cert = $this->cert($this->dirname);
    $client = new LEClient(
      $this->email,
      $this->getModeUrl(),
      $this->getLogLevel(),
      $cert,
      $this->acc
    );
    $account = $client->getAccount();
    // Order.
    $order = $client->getOrCreateOrder($this->basename, $this->domains);
    // Check whether there are any authorizations pending.
    // If that is the case, try to verify the pending authorizations.
    if (!$order->allAuthorizationsValid()) {
      // Get the HTTP challenges from the pending authorizations.
      $pending = $order->getPendingAuthorizations($this->type);
      // Do Acme Challenge.
      if (!empty($pending)) {
        foreach ($pending as $challenge) {
          // Wright challenge.
          if (!$callback) {
            $this->acmeChallenge($challenge);
          }
          else {
            $callback($challenge, $this->extra);
          }
          // Let LetsEncrypt verify this challenge.
          $order->verifyPendingOrderAuthorization($challenge['identifier'], $this->type);
        }
      }
    }
    // Check once more whether all authorizations are valid
    // before we can finalize the order.
    if ($order->allAuthorizationsValid()) {
      // Finalize the order first, if that is not yet done.
      if (!$order->isFinalized()) {
        $order->finalizeOrder();
      }
      // Check whether the order has been finalized before we can get
      // the certificate. If finalized, get the certificate.
      if ($order->isFinalized()) {
        $order->getCertificate();
        file_put_contents($cert['domains'], implode("\n", $this->domains));
        $date = time() + 60 * 60 * 24 * 90;
        $datetime = \Drupal::service('date.formatter')->format($date, 'custom', 'Y-m-d\TH:i:s');
        file_put_contents($cert['expire'], $datetime);
      }
    }
    return implode(", \n", $this->domains);
  }

  /**
   * Get Cert.
   */
  public function sign($basename, array $domains = []) {
    $all = [];
    $this->domains = array_map(function ($domain) {
      $domain = trim($domain);
      return \Drupal::service('idna')->encode($domain);
    }, $domains);
    $this->basename = $basename;
    $this->dirname = $basename;
    // $callback = $this->acmeChallenge();
    $this->signPipe();
    return implode(", \n", $this->domains);
  }

  /**
   * Acme Challenge.
   */
  private function acmeChallenge($challenge, $drupalroot = TRUE) {
    \Drupal::service('module_handler')->alter('letsencrypt_challenge', $challenge, $drupalroot);
    if ($drupalroot) {
      $dir = $this->challenge;
      $this->createDir($this->challenge, FALSE);
      file_put_contents("{$dir}/{$challenge['filename']}", $challenge['content']);
    }
  }

  /**
   * Prepare cert directory.
   */
  private function prepareDir($config) {
    $cdir = getenv("HOME") . "cert";
    if ($dir = $config->get('cert-dir')) {
      if (substr($dir, 0, '2') == '~/') {
        $dir = getenv("HOME") . substr($dir, 2);
      }
      if (substr($dir, 0, '2') == 'private://') {
        $dir = \Drupal::service('file_system')->realpath($dir);
      }
      $cdir = $dir;
    }
    $this->createDir($cdir);
    $this->createDir("$cdir/_account");
    return $cdir;
  }

  /**
   * Cert Path.
   */
  public function cert($base) {
    $ddir = \Drupal::transliteration()->transliterate($this->cdir . "/$base");
    $this->createDir($ddir);
    $cert = [
      "public_key" => "{$ddir}/public.pem",
      "private_key" => "{$ddir}/private.pem",
      "certificate" => "{$ddir}/certificate.crt",
      "fullchain_certificate" => "{$ddir}/fullchain.pem",
      "order" => "{$ddir}/order",
      "domains" => "{$ddir}/domains",
      "expire" => "{$ddir}/expire",
    ];
    return $cert;
  }

  /**
   * Create Directory.
   */
  private function createDir($directory, $protect = TRUE) {
    if (!file_exists($directory)) {
      mkdir($directory, 0777, TRUE);
      if ($protect) {
        file_put_contents("$directory/.htaccess", "order deny,allow\ndeny from all");
      }
    }
  }

}
